@@ -1,6 +1,6 @@
 """Admin module for Django."""
 from django.contrib import admin
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from django_q.conf import Conf
 from django_q.models import Success, Failure, Schedule, OrmQ
