@@ -5,7 +5,6 @@ import pytest
 import redis
 
 from django_q.brokers import get_broker, Broker
-from django_q.compat import range
 from django_q.conf import Conf
 from django_q.humanhash import uuid
 
@@ -45,6 +44,10 @@ def test_redis(monkeypatch):
     broker = get_broker()
     with pytest.raises(Exception):
         broker.ping()
+    monkeypatch.setattr(Conf, 'REDIS', 'redis://127.0.0.1:7799')
+    broker = get_broker()
+    with pytest.raises(Exception):
+        broker.ping()
 
 
 def test_custom(monkeypatch):
@@ -63,7 +66,7 @@ def test_disque(monkeypatch):
     assert broker.info() is not None
     # clear before we start
     broker.delete_queue()
-    # enqueue
+    # async_task
     broker.enqueue('test')
     assert broker.queue_size() == 1
     # dequeue
@@ -127,7 +130,7 @@ def test_ironmq(monkeypatch):
     # clear before we start
     broker.purge_queue()
     assert broker.queue_size() == 0
-    # enqueue
+    # async_task
     broker.enqueue('test')
     # dequeue
     task = broker.dequeue()[0]
@@ -136,7 +139,7 @@ def test_ironmq(monkeypatch):
     assert broker.dequeue() is None
     # Retry test
     # monkeypatch.setattr(Conf, 'RETRY', 1)
-    # broker.enqueue('test')
+    # broker.async_task('test')
     # assert broker.dequeue() is not None
     # sleep(3)
     # assert broker.dequeue() is not None
@@ -180,7 +183,7 @@ def canceled_sqs(monkeypatch):
     assert broker.ping() is True
     assert broker.info() is not None
     assert broker.queue_size() == 0
-    # enqueue
+    # async_task
     broker.enqueue('test')
     # dequeue
     task = broker.dequeue()[0]
@@ -240,7 +243,7 @@ def test_orm(monkeypatch):
     assert broker.info() is not None
     # clear before we start
     broker.delete_queue()
-    # enqueue
+    # async_task
     broker.enqueue('test')
     assert broker.queue_size() == 1
     # dequeue
@@ -297,7 +300,7 @@ def test_mongo(monkeypatch):
     assert broker.info() is not None
     # clear before we start
     broker.delete_queue()
-    # enqueue
+    # async_task
     broker.enqueue('test')
     assert broker.queue_size() == 1
     # dequeue
